// *******************************************************************
// Copyright (C) 2009 Telos Systems, TLS Corp.
// All rights reserved.
//
// No part of this source code or associated documentation may be 
// reproduced or utilized in any form or by any means, electronic or 
// mechanical, without permission in writing from TLS Corp.
// -------------------------------------------------------------------
//      File: APSend.h
// Description: See class description.
//    Author: Ioan L. Rus
//      Date: Nov 18, 2009
// Revisions: 
// -------------------------------------------------------------------
// INI Date        Description
// --- ----------- ---------------------------------------------------
// 
// *******************************************************************
#ifndef _MSG2_H_INCLUDED_
#define _MSG2_H_INCLUDED_

// Desc: Adds name functions to CMsg2 when defined
// Remarks:
// Define USE_INLINE_NAME_METHODS to include inline functions
// providing string identifiers for CMsg2 items in addition to
// the default DWORD tags.
#define USE_INLINE_NAME_METHODS

// Desc: Make all CMsg2 size fields WORD typed.
// Remarks: Define USE_WORD_SIZE to make all sizes used by the message
// WORD sized instead of DWORD sized.  This saves space when 
// the message is converted to a byte stream since all sizes
// take up two bytes instead of four when USE_WORD_SIZE is
// defined.  Note that when USE_WORD_SIZE is defined, a message
// has the following limitations:
//
// - can store a maximum of 65535 items,
// - can be a maximum of 65535 bytes when converted
// to a bytestream.
//
// These limitations should not cause a problem for most situations.
// Undefining USE_WORD_SIZE will allow message sizes to grow
// to 4294967295 bytes.
//
// See Also: CMsg2::GetDataSize(), CMsg2::GetData(), CMsg2::SetData()
#define USE_WORD_SIZE

#ifdef USE_WORD_SIZE

// Desc: Data type for CMsg2 size or count variables
// Remarks:
// CMSG2_SIZE data type is used for size or count variables in CMsg2.
// When USE_WORD_SIZE is defined CMSG2_SIZE is typedef'ed as
// *unsigned short* otherwise it is typedef'ed as *unsigned long*.
typedef unsigned short CMSG2_SIZE;

// Desc: Holds max size for CMsg2 size or count variables
// Remarks:
// CMSG2_SIZE_MAX is defined to be the maximum value for the
// CMSG2_SIZE type.  When USE_WORD_SIZE is defined then CMSG_SIZE_MAX
// is defined to be 0xFFFF otherwise it is defined to be 0xFFFFFFFF.
#define CMSG2_SIZE_MAX 0xFFFF

#else  // USE_WORD_SIZE

typedef unsigned long  CMSG2_SIZE;

#define CMSG2_SIZE_MAX 0xFFFFFFFF

#endif // USE_WORD_SIZE


// Desc: Message class for tag-value pairs.
// Remarks: CMsg2 is a message class designed to store *tag-value pairs*.
// *Tags* are 32-bit unsigned integers that uniquely identify each 
// message item.  A message item can store numeric values 
// (8, 16, and 32 bit), numeric arrays, ASCIIZ strings, or 
// nested messages.
//
// Each message is identified by a 32-bit unsigned integer (see
// SetId() and GetId() methods).  The *message Id* is used to 
// classify messages in an application 
// specific manner.  Often, the message Id identifies the function
// to be carried out by the recipient of the message.  The Id can
// also identify the contents of the message although it is always
// possible to discover the message contents by directly examining
// the message (see GetItemCount(), GetItemTag(), and GetItemType() 
// methods).
//
// Numeric tags are often not convenient to use.  String *names* of
// up to four characters can be used instead of numeric tags for
// most method calls. String names allow for easy recognition of 
// message items and do not rely on a global enumeration.  They
// are easier to document as well.  There are two ways to use names 
// in place of numeric tags:
//
// - First, by calling methods that accept names as parameters, or
// - Second, by using one of the GetTag() methods to convert a
// name to a tag, or a prefix and an index to a tag.
//
// Care should be exercised when using numeric and string identifiers
// in the same message to prevent unexpected name collisions.  For 
// example, "MSG2" and 0x4D534732 refer to the same item.  See the
// GetTag() method for more details on how names are converted to tags.

unsigned long  NameToTag  (const char *pszName);
char          *DwordToName(unsigned long lValue,char *pszName,const char *pszPrefix=0);

class CMsg2  
{
public:
  // Data types that can be stored in a message.
  enum 
  {
    TypInvalid=0,     // Zero is not a valid data type.
    TypDword,         // 4-byte value.
    TypString,        // ASCIIZ string.
    TypByteArray,     // Array of byte values.
    TypWordArray,     // Array of word (2-byte) values.
    TypDwordArray,    // Array or dword (4-byte) values.
    TypMsg,           // Nested CMsg2.
    TypByte,          // Single byte value.
    TypWord           // Word (2-byte) value.
  };

private:
  // Item node type
  struct Node
  {
    unsigned long  m_ulName;
    unsigned char  m_eType;
    CMSG2_SIZE     m_nArrayCount;

    union Value
    {
      long            m_lNumber;
      char           *m_pString;
      unsigned char  *m_pByteArray;
      unsigned short *m_pWordArray;
      unsigned long  *m_pDwordArray;
      CMsg2       *m_pMsg;

      Value(long lValue);
    } m_Value;

    Node();
    void DeleteValue();
  } *m_pNodeArray;

  unsigned long m_ulMessageId; // Message id.  Defined here for easy access to message type.

  CMSG2_SIZE    m_nCount;      // Number of items in message

  unsigned int  m_nSize;       // Size of the message item array
  unsigned int  m_nGrowBy;     // Grow count when we run out of item space in array

  unsigned int  m_nIndex; // 

  // Internal function
  Node *FindItem(unsigned long ulName);
  // Internal function
  Node *AddItem(unsigned long ulName);

public:
  CMsg2(unsigned int nInitialSize=16,unsigned int nGrowBy=16);
  CMsg2(const char *pData,unsigned long cbData,unsigned int nGrowBy=16);
  CMsg2(const CMsg2 &msg);
  virtual ~CMsg2();

  CMsg2 &operator =(const CMsg2 &msg);
  CMsg2 &operator +=(const CMsg2 &msg);
  void Copy(const CMsg2 *pMsg);

  static unsigned char GetVer();

  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
  // Access message as a byte stream
  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

  // Get the packed message size.
  CMSG2_SIZE GetDataSize() const;

  // Copy the message to a data buffer.
  CMSG2_SIZE GetData(char *pData) const;
  // Load new message contents by reading them from a data buffer.
  CMSG2_SIZE SetData(const char *pData,unsigned long cbData);

  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
  // Set/Get message identifier value
  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

  // Set the message id.
  void SetId(unsigned long ulMessageTag);
  // Get the message id.
  unsigned long GetId() const;

  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
  // Store simple values
  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

  // Set number value (1-byte)
  bool SetByte(unsigned long ulTag,unsigned char ucNum);
  // Set number value (2-byte)
  bool SetWord(unsigned long ulTag,unsigned short usNum);
  // Set number value (4-byte)
  bool SetDword(unsigned long ulTag,unsigned long ulNum);
  // Set bool
  bool SetBool(unsigned long ulTag,bool bValue);
  // Set string value
  bool SetString(unsigned long ulTag,const char *pszValue);

  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
  // Retrieve simple values
  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

  // Return a 1-byte number value.
  unsigned char  GetByte(unsigned long ulTag);
  // Return a 2-byte number value.
  unsigned short GetWord(unsigned long ulTag);
  // Return a 4-byte number value.
  unsigned long  GetDword(unsigned long ulTag);
  // Return any numeric value.
  unsigned long  GetNumber(unsigned long ulTag);
  // Return true/false
  bool           GetBool(unsigned long ulTag);
  // Return a pointer to a string.
  const char    *GetString(unsigned long ulTag);


  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
  // Store/Retrieve nested messages
  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

  bool   SetMsg(unsigned long ulTag,const CMsg2 *pMsg);
  CMsg2 *GetMsg(unsigned long ulTag);

  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
  // Store Arrays
  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

  // Set short array
  bool SetShortArray(const char *pszName,unsigned int nItems,short *pData=0);
  // Set long array
  bool SetLongArray (const char *pszName,unsigned int nItems,long *pData=0);
  // Set byte array
  bool SetByteArray(unsigned long ulTag,unsigned int nItems,unsigned char *pData=0);
  // Set word array
  bool SetWordArray(unsigned long ulTag,unsigned int nItems,unsigned short *pData=0);
  // Set double word array
  bool SetDwordArray(unsigned long ulTag,unsigned int nItems,unsigned long *pData=0);

  // Set byte array item
  bool SetByteArrayItem(unsigned long ulTag,unsigned int nIndex,unsigned char bData);
  // Set word array item
  bool SetWordArrayItem(unsigned long ulTag,unsigned int nIndex,unsigned short usData);
  // Set double word array item
  bool SetDwordArrayItem(unsigned long ulTag,unsigned int nIndex,unsigned long ulData);

  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
  // Retrieve Arrays
  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

  // Get number of items in array
  // Returns: The number of items in array
  CMSG2_SIZE GetArrayCount(unsigned long ulTag);

  // Get pointer to byte array.
  unsigned char  *GetByteArray(unsigned long ulTag);
  // Get pointer to word array.
  unsigned short *GetWordArray(unsigned long ulTag);
  // Get pointer to double word array.
  unsigned long  *GetDwordArray(unsigned long ulTag);
// Get pointer to short array.
  short          *GetShortArray(const char *pszName);
// Get pointer to long array.
  long           *GetLongArray (const char *pszName);

  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
  // Utility functions
  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

  unsigned int GetItemCount() const;
  bool GetItemTag(unsigned int nIndex,unsigned long &ulTag) const;
  bool GetItemName(unsigned int nIndex,char *pszName) const;

  // Returns true if name exists in message.
  bool Exists(unsigned long ulTag);
  // Removes item from message.
  bool Remove(unsigned long ulTag);
  // Returns item type.
  int  GetItemType(unsigned long ulTag);
  // Remove all items from message
  void Empty();

public:
  static unsigned long GetTag(const char *pszPrefix,unsigned int uIndex);
  static unsigned long GetTag(const char *pszName);
  static char         *GetName(unsigned long ulTag,char *pszName);

#ifdef USE_INLINE_NAME_METHODS
  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
  // Test message identifier value
  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

  inline void SetId(const char *pszName);

  // Check the message ID against specified name
  inline bool TestId(const char *pszName);

  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
  // Store simple values
  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
  // Set number value
  bool SetNumber(const char *pszName,long lNum);

  inline bool SetByte(const char *pszName,unsigned char ucNum);
  inline bool SetWord(const char *pszName,unsigned short usNum);
  inline bool SetDword(const char *pszName,unsigned long ulNum);
  inline bool SetBool(const char *pszName,bool bValue);
  inline bool SetString(const char *pszName,const char *pszValue);

  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
  // Retrieve simple values
  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

  inline unsigned char  GetByte(const char *pszName);
  inline unsigned short GetWord(const char *pszName);
  inline unsigned long  GetDword(const char *pszName);
  inline long           GetNumber(const char *pszName);
  inline bool           GetBool(const char *pszName);
  inline const char    *GetString(const char *pszName);

  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
  // Store/Retrieve nested messages
  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

  inline bool  SetMsg(const char *pszName,const CMsg2 *pMsg);
  inline CMsg2 *GetMsg(const char *pszName);

  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
  // Store Arrays
  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

  inline bool SetByteArray (const char *pszName,unsigned int nItems,unsigned char *pData=0);
  inline bool SetWordArray (const char *pszName,unsigned int nItems,unsigned short *pData=0);
  inline bool SetDwordArray(const char *pszName,unsigned int nItems,unsigned long *pData=0);

  inline bool SetByteArrayItem (const char *pszName,unsigned int nIndex,unsigned char bData);
  inline bool SetWordArrayItem (const char *pszName,unsigned int nIndex,unsigned short usData);
  inline bool SetDwordArrayItem(const char *pszName,unsigned int nIndex,unsigned long ulData);

  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
  // Retrieve Arrays
  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

  inline CMSG2_SIZE      GetArrayCount(const char *pszName);

  inline unsigned char  *GetByteArray (const char *pszName);
  inline unsigned short *GetWordArray (const char *pszName);
  inline unsigned long  *GetDwordArray(const char *pszName);

  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
  // Utility functions
  //\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

  inline bool Exists(const char *pszName);
  inline bool Remove(const char *pszName);
  inline int  GetItemType(const char *pszName);
#endif // USE_INLINE_NAME_METHODS
};


#ifdef USE_INLINE_NAME_METHODS
void CMsg2::SetId(const char *pszName  // Name for message (up to four characters)
                  )
{
  SetId(GetTag(pszName));
}

//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
// Test message identifier value
//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

// Check the message ID against specified name
// Returns: true if the message ID matches the specified name, 
// false otherwise
bool CMsg2::TestId(const char *pszName // Pointer to name holding ID to test
                   )
{
  return GetId()==GetTag(pszName);
}

//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
// Store simple values
//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

bool CMsg2::SetByte(const char *pszName, // Name for message (up to four characters)
                    unsigned char ucNum)
{
  return SetByte(GetTag(pszName),ucNum);
}

bool CMsg2::SetWord(const char *pszName, // Name for message (up to four characters)
                    unsigned short usNum)
{
  return SetWord(GetTag(pszName),usNum);
}

bool CMsg2::SetDword(const char *pszName, // Name for message (up to four characters)
                    unsigned long ulNum)
{
  return SetDword(GetTag(pszName),ulNum);
}

bool CMsg2::SetBool(const char *pszName, // Name for message (up to four characters)
                    bool bValue)
{
  return SetBool(GetTag(pszName),bValue);
}

bool CMsg2::SetString(const char *pszName, // Name for message (up to four characters)
                    const char *pszValue)
{
  return SetString(GetTag(pszName),pszValue);
}


//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
// Retrieve simple values
//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

unsigned char CMsg2::GetByte(const char *pszName // Name for message (up to four characters)
                             )
{
  return GetByte(GetTag(pszName));
}

unsigned short CMsg2::GetWord(const char *pszName // Name for message (up to four characters)
                              )
{
  return GetWord(GetTag(pszName));
}

unsigned long CMsg2::GetDword(const char *pszName /* Name for message (up to four characters) */)
{
  return GetDword(GetTag(pszName));
}

long CMsg2::GetNumber(const char *pszName /* Name for message (up to four characters) */)
{
  return GetNumber(GetTag(pszName));
}

bool CMsg2::GetBool(const char *pszName /* Name for message (up to four characters) */)
{
  return GetBool(GetTag(pszName));
}

const char *CMsg2::GetString(const char *pszName /* Name for message (up to four characters) */)
{
  return GetString(GetTag(pszName));
}

//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
// Store/Retrieve nested messages
//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

bool CMsg2::SetMsg(const char *pszName /* Name for message (up to four characters) */,const CMsg2 *pMsg)
{
  return SetMsg(GetTag(pszName),pMsg);
}

CMsg2 *CMsg2::GetMsg(const char *pszName /* Name for message (up to four characters) */)
{
  return (CMsg2*)GetMsg(GetTag(pszName));
}

//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
// Store Arrays
//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

bool CMsg2::SetByteArray (const char *pszName /* Name for message (up to four characters) */,unsigned int nItems,unsigned char *pData)
{
  return SetByteArray(GetTag(pszName),nItems,pData);
}

bool CMsg2::SetWordArray (const char *pszName, /* Name for message (up to four characters) */unsigned int nItems,unsigned short *pData)
{
  return SetWordArray(GetTag(pszName),nItems,pData);
}

bool CMsg2::SetDwordArray(const char *pszName /* Name for message (up to four characters) */,unsigned int nItems,unsigned long *pData)
{
  return SetDwordArray(GetTag(pszName),nItems,pData);
}

bool CMsg2::SetByteArrayItem (const char *pszName /* Name for message (up to four characters) */,unsigned int nIndex,unsigned char bData)
{
  return SetByteArrayItem(GetTag(pszName),nIndex,bData);
}

bool CMsg2::SetWordArrayItem (const char *pszName /* Name for message (up to four characters) */,unsigned int nIndex,unsigned short usData)
{
  return SetWordArrayItem(GetTag(pszName),nIndex,usData);
}

bool CMsg2::SetDwordArrayItem(const char *pszName /* Name for message (up to four characters) */,unsigned int nIndex,unsigned long ulData)
{
  return SetDwordArrayItem(GetTag(pszName),nIndex,ulData);
}

//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
// Retrieve Arrays
//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

CMSG2_SIZE CMsg2::GetArrayCount(const char *pszName /* Name for message (up to four characters) */)
{
  return GetArrayCount(GetTag(pszName));
}

unsigned char *CMsg2::GetByteArray (const char *pszName /* Name for message (up to four characters) */)
{
  return GetByteArray(GetTag(pszName));
}

unsigned short *CMsg2::GetWordArray (const char *pszName /* Name for message (up to four characters) */)
{
  return GetWordArray(GetTag(pszName));
}

unsigned long *CMsg2::GetDwordArray(const char *pszName /* Name for message (up to four characters) */)
{
  return GetDwordArray(GetTag(pszName));
}

//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
// Utility functions
//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

bool CMsg2::Exists(const char *pszName /* Name for message (up to four characters) */)
{
  return Exists(GetTag(pszName));
}

bool CMsg2::Remove(const char *pszName /* Name for message (up to four characters) */)
{
  return Remove(GetTag(pszName));
}

int CMsg2::GetItemType(const char *pszName /* Name for message (up to four characters) */)
{
  return GetItemType(GetTag(pszName));
}
#endif // USE_INLINE_NAME_METHODS


#endif // _MSG2_H_INCLUDED_
